######################################################################
# This file was *autogenerated* from the file conjecture.sage.
######################################################################
######################################################
# Code for obtaining data about the conjecture
######################################################

if not os.path.exists('cache'):
    os.makedirs('cache')

def reducible_primes(M, bound):
    """
    Given a modular symbols space M, find the all primes
    up to bound such that the charpoly of a_p is reducible.
    """
    return [p for p in prime_range(bound) if not \
            M.hecke_polynomial(p).is_irreducible()]


def first_with_degree(d, k):
    """
    Find the lowest-level modular symbols space corresponding to a
    newform of degree d and weight k.
    """
    filename = 'cache/%s-%s.sobj'%(d,k)
    if os.path.exists(filename):
        return load(filename)
    N = Integer(1)
    while True:
        M = ModularSymbols(N, k, sign=Integer(1))
        S = M.cuspidal_subspace().new_subspace()
        D = S.decomposition()
        for A in D:
            if A.dimension() == d:
                A.filename = filename
                A.save(filename)
                return A
        N += Integer(1)

def first_n_with_degree(n, d, k):
    """
    Find the lowest-level modular symbols space corresponding to a
    newform of degree d and weight k.
    """
    filename = 'cache/several-%s-%s-%s.sobj'%(n, d,k)
    if os.path.exists(filename):
        return load(filename), filename
    N = Integer(1)
    v = []
    while len(v) < n:
        M = ModularSymbols(N, k, sign=Integer(1))
        S = M.cuspidal_subspace().new_subspace()
        D = S.decomposition()
        for A in D:
            if A.dimension() == d:
                v.append(A)
        N += Integer(1)
    save(v, filename)
    return v, filename

def make_table(bound,verbose=False):
    for k in [Integer(2),Integer(4),Integer(6),Integer(8),Integer(10),Integer(12)]:
        for d in [Integer(2),Integer(3),Integer(4)]:
            M = first_with_degree(d, k)
            v = reducible_primes(M, bound)
            M.save(M.filename) # cache computed charpolys so far
            print "%s & %s & %s & %s \\\\"%(
                k, d, M.level(), ', '.join([str(x) for x in v]))

def make_several_table(n,d,k,bound,verbose=False):
    w, filename = first_n_with_degree(n, d, k)
    for M in w:
        v = reducible_primes(M, bound)
        save(w, filename)
        print "%s & %s & %s & %s \\\\"%(
            k, d, M.level(), ', '.join([str(x) for x in v]))

def make_several_table_counts(n,d,k,bound,verbose=False):
    w, filename = first_n_with_degree(n, d, k)
    for M in w:
        v = reducible_primes(M, bound)
        save(w, filename)
        print "%s & %s & %s & %s \\\\"%(
            k, d, M.level(), len(v))


def level389(bound):
    filename = '389.sobj'
    if os.path.exists(filename):
        m = load(filename)
    else:
        # Just T2 is enough to decompose this space.
        m = ModularSymbols(Integer(389),Integer(2),sign=Integer(1)).cuspidal_subspace().decomposition(Integer(2))
    i = Integer(0)
    t = cputime()
    for p in primes(bound):
        m[Integer(0)].hecke_matrix(p)
        i += Integer(1)
        if i % Integer(50) == Integer(0):
            print "Saving... (p=%s out of %s, time so far=%s)"%(p, bound, cputime(t))
            save(m, filename)
    save(m, filename)
    for a in m:
        v = [p for p in primes(bound) if not a.hecke_polynomial(p).is_irreducible()]
        print "2 & %s & 389 & %s \\\\"%(a.dimension(), ', '.join([str(x) for x in v]))
    save(m, filename)


